﻿using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.Networking;
//using UnityEditor;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

public class GameManager : NetworkBehaviour {
	public static GameManager Instance;
	
	public static List<Player> blueTeam = new List<Player>();
	public static List<Player> redTeam = new List<Player>();
	
	public static Vector3 blueLeft = new Vector3(10,3,65), blueRight = new Vector3(40,3,65);
	public static Vector3 redLeft = new Vector3(10,3,25), redRight = new Vector3(40,3,25);
	public static Vector3 ballPos = new Vector3(25,10,45);
	
	public SandImpact sandImpact;
	public SandShock sandShock;
	public SandWall sandWall;
	
	[SyncVar]
	public int numBlue = 0;
	[SyncVar]
	public int numRed = 0;
	
	[SyncVar]
	public int scoreBlue = 0;
	[SyncVar]
	public int scoreRed = 0;
	
	[SyncVar]
	public float sec = 0;
	
	
	public static Vector3 GetPos(int team) {
		if(team == 1)
			return Vector3.Lerp(blueLeft, blueRight, UnityEngine.Random.value);
		else
			return Vector3.Lerp(redLeft, redRight, UnityEngine.Random.value);
		// return ballPos;
	}
	
	public static bool InsideGameArea(Vector3 position) {
		if(position.x < 0 || position.x > OWTerrain.sizeW * OWTerrain.pointLength)
			return false;
		if(position.x < 0 || position.x > OWTerrain.sizeH * OWTerrain.pointLength)
			return false;

		return true;
	}
	
	public void Restart() {
		scoreBlue = scoreRed = 0;
		sec = 0;
		ResetPosition();
	}
	
	public void Goal(int team) {
		if(!isServer)
			return;
		
		if(team == 1)
			GoalBlue();
		else
			GoalRed();
	}
	
	public void GoalRed() {
		scoreRed += 1;
		ShowScore.Instance.Show("Red Score!!!");
		ResetPosition();
	}
	
	public void GoalBlue() {
		scoreBlue += 1;
		ShowScore.Instance.Show("Blue Score!!!");
		ResetPosition();
	}
	
	public void ResetPosition() {
		Ball.Instance.RpcSetPosition(ballPos);
		for(int i = 0; i< blueTeam.Count; i++) {
			blueTeam[i].RpcSetPosition(Vector3.Lerp(blueLeft, blueRight, (i+1f)/(blueTeam.Count+1f)));
		}
		
		for(int i = 0; i< redTeam.Count; i++) {
			redTeam[i].RpcSetPosition(Vector3.Lerp(redLeft, redRight, (i+1f)/(redTeam.Count+1f)));
		}
	}
	
	void Awake() {
		Instance = this;
	
		if(GameCore.Instance == null)
			SceneManager.LoadScene("GameMenu");
	}
	
	
	void Update() {
		if(!isServer)
			return;
	
		CleanTeam(blueTeam);
		CleanTeam(redTeam);
		
		numBlue = blueTeam.Count;
		numRed = redTeam.Count;
		
		sec += Time.deltaTime;
		
		if(Input.GetKeyDown(KeyCode.F1))
			Restart();
	}
	
	private void CleanTeam(List<Player> plist) {
		for(int i =0; i< plist.Count; i++) {
			if(plist[i] == null) {
				plist.RemoveAt(i);
				i--;
			}
		}
	}
	
	
	
	[ClientRpc]
	public void RpcShockGround(Vector3 pos, float force, float radios, float liftTime) {
		// OWTerrain.Instance.SetWave(pos,force,radios);
		var ss = GameObject.Instantiate(sandShock, pos, Quaternion.identity) as SandShock;
		ss.liftTime = liftTime;
		ss.force = force;
		ss.radios = radios;
	}
	
	[ClientRpc]
	public void RpcSandImpact(Vector3 pos, Quaternion direction) {
		var position = pos;
		position.y = 0;
		GameObject.Instantiate(sandImpact, position, direction);
	}
	
	[ClientRpc]
	public void RpcSandWall(Vector3 pos, Quaternion direction) {
		GameObject.Instantiate(sandWall, pos, direction);
	}
}
