﻿using UnityEngine;
using UnityEngine.Networking;
//using UnityEditor;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

// [Serializable]
// public class Impact {
	// public float impactForcePM = 0.5f;
	// public float impactRadPM = 0.3f;
// }

[RequireComponent(typeof(Rigidbody))]
public class OWObject : NetworkBehaviour {
	public const float bumpMax = 10;
	public Rigidbody rBody;
	
	public float slopeForce = 10;
	public float heightOffset = 0;
	public float shockJumpDamp = 0.25f;
	
	// public Impact impact = new Impact();
	
	public float impactForce = -1.5f;
	public float impactRadios = 3f;
	
	public AnimationCurve shockCurve = AnimationCurve.Linear(0,0,1,0);
	
	public bool isOnGround {
		get {return groundHeight < 0.1f ; }
	}
	
	[SyncVar]
	public Vector3 moveVelocity = Vector3.zero;
	
	public Transform rotateTransform;
	
	public float groundHeight = 10;
	
	protected Vector3 lastPosition;
	protected Vector3 velocity = Vector3.zero;
	
	protected Vector3 externalAcc = Vector3.zero;
	
	protected float shockTime = 0;
	protected bool lastHitGround = false;
	
	protected float syncInterval = 0;
	protected Quaternion syncRot = Quaternion.identity;
	protected List<Vector3> syncPosList = new List<Vector3>();
	protected Vector3 syncPos = Vector3.zero, lastSyncPos = Vector3.zero;
	
	protected float collideTime = 0f;
	
	[Command]
	public void CmdSetPosition(Vector3 pos) {
		RpcSetPosition(pos);
	}
	
	[ClientRpc]
	public void RpcSetPosition(Vector3 pos) {
		SetPosition(pos);
	}
	
	protected virtual void SetPosition(Vector3 pos) {
		transform.position = lastPosition = syncPos = pos;
		velocity = Vector3.zero;
		syncPosList.Clear();
		externalAcc = Vector3.zero;
		moveVelocity = Vector3.zero;
		rBody.velocity = Vector3.zero;
		if(hasAuthority)
			OnSetPosition();
	}
	
	protected virtual void OnSetPosition() {
		
	}
	
	[Command]
	public void CmdSetVelocity(Vector3 vel) {
		moveVelocity = vel;
	}
	
	[Command]
	public void CmdSyncTrans(Vector3 position, Quaternion rotation) {
		RpcSyncTrans(position, rotation);
	}
	
	[Command]
	public void CmdShockGround(Vector3 pos, float force, float radios, float liftTime) {
		GameManager.Instance.RpcShockGround(pos,force,radios,liftTime);
	}
	
	public Vector3 ModifyPosition(Vector3 point) {
		var position = point;
		position.x = Mathf.Clamp(position.x,0,OWTerrain.sizeW * OWTerrain.pointLength);
		position.z = Mathf.Clamp(position.z,0,OWTerrain.sizeH * OWTerrain.pointLength);
		return position;
	}
	
	[ClientRpc]
	protected void RpcSyncTrans(Vector3 position, Quaternion rotation) {
		if(hasAuthority)
			return;
		syncRot = rotation;
		
		if(syncPosList.Count == 0)
			lastSyncPos = transform.position;
		syncPosList.Add(position);
	}
	
	
	// Use this for initialization
	protected virtual void Start () {
		lastPosition = transform.position;
		syncPos = lastPosition;
		
		if(!hasAuthority) {
			rBody.isKinematic = true;
		}
		
	}
	
	protected virtual void Update() {
		if(hasAuthority) {
			ObjectUpdate();
			SendPos();
		}
		else
			SyncPos();
	}
	
	protected virtual void ObjectUpdate() {
		if(lastHitGround != isOnGround) {
			// Debug.Log(name + " ObjectUpdate " +  isOnGround);
			
			if(isOnGround)
				HitGround();
			lastHitGround = isOnGround;
			GroundChange();
		}
	}
	
	protected virtual void GroundChange() {
		
	}
	
	protected virtual void SendPos() {
		syncInterval -= Time.deltaTime;
		bool isSync = false;
		
		if(syncInterval > 0)
			return;
			
		CmdSetVelocity(velocity);
		
		if(Vector3.Distance(syncPos, transform.position) > 0.025f)
			isSync = true;
		else if(Quaternion.Angle(rotateTransform.rotation, syncRot) > 0.5f)
			isSync = true;
		
		syncInterval = 0.1f;
		
		if(!isSync)
			return;
		
		CmdSyncTrans(transform.position, rotateTransform.rotation);
		syncPos = transform.position;
		syncRot = rotateTransform.rotation;
	}
	
	protected virtual void SyncPos() {
		rotateTransform.rotation = Quaternion.RotateTowards(rotateTransform.rotation, syncRot, 360 * Time.deltaTime);
		
		
		if(syncPosList.Count > 0) {
			// syncPos = Vector3.Lerp(syncPos, syncPosList[0], 12f*Time.deltaTime);
			
			float speed = Mathf.Lerp(5.5f,13.5f, syncPosList.Count/5f);
			
			syncPos = Vector3.MoveTowards(syncPos, syncPosList[0],
				Vector3.Distance(lastSyncPos, syncPosList[0]) * speed * Time.deltaTime);
			
			if(Vector3.Distance(syncPos, syncPosList[0]) < 0.05f) {
				lastSyncPos = syncPosList[0];
				syncPosList.RemoveAt(0);
			}
				
			var pos = ModifyPosition(syncPos);
			pos.y = Mathf.Max(pos.y, OWTerrain.Instance.GetTerrainPoint(pos).point.y + heightOffset);
			transform.position = pos;
		}
	}
	
	protected virtual void FixedUpdate() {
		if(hasAuthority)
			ObjectFixedUpdate();
	}
	
	protected virtual void OnCollisionEnter(Collision collision) {
		if(Time.time - collideTime < 0.15f)
			return;
		
		if(collision.collider.gameObject.tag == "Bounce") {
			var pos = collision.collider.transform.position;
			rBody.AddForceAtPosition((transform.position - pos).normalized * 5f, pos, ForceMode.VelocityChange);
			
			collideTime = Time.time;
			Collision();
		}
	}
	
	protected virtual void Collision() {
	
	}
	
	protected virtual void HitGround() {
		if(velocity.y > 0)
			return;
		if(Time.time - shockTime < 2.5f)
			return;
			
		// Debug.Log(name + " HitGround");
	
		// OWTerrain.Instance.SetWave(transform.position,-1.5f, 5, shockCurve);
		// OWTerrain.Instance.SetWave(transform.position,-3f, 5);
		CmdShockGround(transform.position,impactForce, impactRadios,0.05f);
		shockTime = Time.time;
	}
	
	protected virtual void ObjectFixedUpdate () {
		var tp = OWTerrain.Instance.GetTerrainPoint(transform.position);
		// var yOffset = 0f;
		velocity = rBody.velocity;
		
		if(transform.position.y < tp.point.y + heightOffset) {
			// yOffset = tp.point.y - transform.position.y;
			transform.position = ModifyPosition(tp.point + Vector3.up * heightOffset);
			
			var slope = tp.normal;
			slope.y = 0;
			
			velocity = (transform.position - lastPosition) * 50;
			velocity.y = Mathf.Clamp(velocity.y * shockJumpDamp, -bumpMax, bumpMax);
			
			rBody.AddForce(slope * slopeForce, ForceMode.Acceleration);
		}
		else {
			transform.position = ModifyPosition(transform.position);
			velocity = (transform.position - lastPosition) * 50;
		}
		
		groundHeight = transform.position.y - tp.point.y - heightOffset;
		RaycastHit hit;
		
		if(Physics.Raycast(transform.position + Vector3.up * 0.1f, Vector3.up*-1, out hit, 1)) {
			groundHeight = Mathf.Min(groundHeight, hit.distance - 0.1f);
		}
		
		rBody.AddForce(externalAcc, ForceMode.Acceleration);
		externalAcc = Vector3.zero;
		
		rBody.velocity = velocity;
		
		lastPosition = transform.position;
	}
}
