﻿using UnityEngine;
using UnityEngine.Networking;
//using UnityEditor;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

public class Player : OWObject {

	public const float sandwalkTime = 3f;
	public const float sandwalkSpeed = 20f;
	
	
	public float moveSpeed = 6;
	public float rtSpeed = 360;
	public float maxAcc = 8;
	public float jumpAcc = 5;
	
	[SyncVar]
	public int team = -1;
	
	public Color color1, color2;
	
	public Transform nameText;
	public Renderer render;
	
	public Animator animator;
	public Transform playerTransform;
	
	public AudioSource walk;
	public ParticleSystem sand1, sand2;
	
	protected TextMesh nameTextMesh;
	
	[SyncVar]
	public string playerName = "";
	
	[Command]
	public void CmdSetTeam(int team) {
		if(team == 1)
			GameManager.blueTeam.Add(this);
		else
			GameManager.redTeam.Add(this);
		
		RpcSetTeam(team);
	}
	
	
	
	protected override void OnSetPosition() {
		var angle = team == 1 ? -180 : 0;
		Quaternion q = Quaternion.Euler(0,angle,0);
		rotateTransform.rotation = q;
		Control.Instance.transform.rotation = q;
	}
	
	[ClientRpc]
	public void RpcSetTeam(int team) {
		this.team = team;
		
		Color c = team == 1?color1:color2;
		render.material.SetColor("_Color",c);
		render.material.SetColor("_EmissionColor",c);
		
		SetPosition(GameManager.GetPos(team));
		
		if(hasAuthority) {
			Control.Instance.Begin();
			rBody.isKinematic = false;
			rBody.velocity = Vector3.zero;
		}
		
		
	}
	
	[Command]
	public void CmdFireSandImpact(Vector3 pos, Quaternion rot) {
		GameManager.Instance.RpcSandImpact(pos, rot);
	}
	
	[Command]
	public void CmdSandJet(Vector3 pos) {
		GameManager.Instance.RpcShockGround(pos,8,3,0.25f);
	}
	
	[Command]
	public void CmdSandWalk() {
		RpcSandWalk();
	}
	
	[Command]
	public void CmdSandWall(Vector3 pos, Quaternion rot) {
		GameManager.Instance.RpcSandWall(pos, rot);
	}
	
	public void Run(Vector3 direction) {
		if(!isOnGround)
			return;
		Move(direction, moveSpeed);
	}
	
	public void SandWalk(Vector3 movement) {
		var acc = movement - velocity;
		externalAcc += acc;
	}
	
	public void Move(Vector3 direction, float speed) {
		var dir = direction.normalized;
		
		var acc = dir * speed - velocity;
		
		acc = acc.normalized * Mathf.Min(acc.magnitude, maxAcc);
		externalAcc += acc;
	}
	
	public void Jump() {
		// Debug.Log("Jump " + isOnGround);
		
		if(!isOnGround)
			return;
		
		rBody.AddForce(Vector3.up * jumpAcc, ForceMode.VelocityChange);
	}
	
	public void Face(Vector3 direction) {
		var dir = direction;
		dir.y = 0;
		dir = dir.normalized;
		if(dir.magnitude < 0.05f)
			return;
		playerTransform.rotation = Quaternion.RotateTowards(playerTransform.rotation, Quaternion.LookRotation(dir), rtSpeed * Time.deltaTime);
	}
	
	[ClientRpc]
	protected void RpcSandWalk() {
		StopAllCoroutines();
		StartCoroutine(ISandWalk());
	}
	
	protected IEnumerator ISandWalk() {
		float t = 0;
		
		while( t < sandwalkTime) {
			t += Time.deltaTime;
			
			OWTerrain.Instance.SetWave(transform.position - transform.forward,transform.position.y + 0.5f, 2f);
			
			yield return null;
		}
		
	}
	
	[Command]
	protected void CmdSetName(string pname) {
		playerName = pname;
	}
	
	[ClientRpc]
	protected void RpcOnGround(bool state) {
		if(state) {
			walk.Play();
			sand1.Play();
		}
		else {
			walk.Stop();
			sand1.Stop();
		}
	}
	
	protected override void GroundChange() {
		RpcOnGround(lastHitGround);
	}
	
	
	protected override void Start() {
		base.Start();
		
		// nameTextMesh = nameText.GetComponent<TextMesh>();
		
		if(hasAuthority) {
			Control.controlTarget = this;
			// transform.position = new Vector3(25,0,30);
			CmdSetName(GameCore.Instance.playerName);
		}
	}
	
	protected override void Update() {
		base.Update();
		
		// if(playerName == "")
			// nameTextMesh.text = playerName;
		
		float move = moveVelocity.magnitude / 6f;
		if(move < 0.1f)
			move = 0;
		
		animator.SetFloat("MoveSpeed",move);
		
		// if(Control.Instance != null)
			// nameText.rotation = Control.Instance.cam.transform.rotation;
			
		
	}
	
}
